@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Base;
 
-our $DATE = '2016-07-20'; # DATE
-our $VERSION = '1.56'; # VERSION
+our $DATE = '2016-08-03'; # DATE
+our $VERSION = '1.57'; # VERSION
 
 use 5.010001;
 use strict;
@@ -1442,7 +1442,7 @@ Perinci::CmdLine::Base - Base class for Perinci::CmdLine{::Classic,::Lite}
 
 =head1 VERSION
 
-This document describes version 1.56 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2016-07-20.
+This document describes version 1.57 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2016-08-03.
 
 =head1 DESCRIPTION
 
@@ -2192,12 +2192,26 @@ user's home and then system location. On Unix, it's C<< [ "$ENV{HOME}/.config",
 $ENV{HOME}, "/etc"] >>. If $ENV{HOME} is empty, getpwuid() is used to get home
 directory entry.
 
-=head2 config_filename => str|array[str]
+=head2 config_filename => str|array[str]|array[hash]
 
 Configuration filename(s). The default is C<< program_name . ".conf" >>. For
 example, if your program is named C<foo-bar>, config_filename will be
 C<foo-bar.conf>.
 
+You can specify an array of filename strings, which will be checked in order,
+e.g.: C<< ["myapp.conf", "myapp.ini"] >>.
+
+You can also specify an array of hashrefs, for more complex scenario. Each hash
+can contain these keys: C<filename>, C<section>. For example:
+
+ [
+     {filename => 'mysuite.conf', section=>'myapp1'},
+     {filename => 'myapp1.conf'}, # section = GLOBAL (default)
+ ]
+
+This means, configuration will be searched in C<mysuite.conf> under the section
+C<myapp1>, and then in C<myapp1.conf> in the default/global section.
+
 =head2 skip_format => bool
 
 If set to 1, assume that function returns raw text that need not be translated,
